// COpenGLView.cpp : implementation of the COpenGLView class
//

#include "stdafx.h"
#include "OpenGL View Class.h"

#include "OpenGL View ClassDoc.h"
#include "COpenGLView.h"
#include "math.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



const char* const COpenGLView::_ErrorStrings[]= {
				{"No Error"},					// 0
				{"Unable to get a DC"},			// 1
				{"ChoosePixelFormat failed"},	// 2
				{"SelectPixelFormat failed"},	// 3
				{"wglCreateContext failed"},	// 4
				{"wglMakeCurrent failed"},		// 5
				{"wglDeleteContext failed"},	// 6
				{"SwapBuffers failed"},			// 7

		};

/////////////////////////////////////////////////////////////////////////////
// COpenGLView

IMPLEMENT_DYNCREATE(COpenGLView, CView)

BEGIN_MESSAGE_MAP(COpenGLView, CView)
	//{{AFX_MSG_MAP(COpenGLView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_ERASEBKGND()
	ON_WM_SIZE()
	ON_COMMAND(ID_ITEM_TORUS, OnItemTorus)
	ON_COMMAND(ID_ITEM_SPHERE, OnItemSphere)
	ON_COMMAND(ID_ITEM_COIL, OnItemCoil)
	ON_COMMAND(ID_ITEM_GOBLET, OnItemGoblet)
	ON_COMMAND(ID_ITEM_CYLINDER, OnItemCylinder)
	ON_COMMAND(ID_ITEM_BILINEAR, OnItemBilinear)
	ON_COMMAND(ID_QUADRICS_ELLIPSEOID, OnQuadricsEllipseoid)
	ON_COMMAND(ID_QUADRICS_HYPERBOLOID, OnQuadricsHyperboloid)
	ON_COMMAND(ID_SUPERQUADRICS_SUPERELLIPSEOID, OnSuperquadricsSuperellipseoid)
	ON_COMMAND(ID_SUPERQUADRICS_SUPERTOROID, OnSuperquadricsSupertoroid)
	ON_COMMAND(ID_ITEM_ANIMATE, OnItemAnimate)
	ON_COMMAND(ID_ITEM_ROTATE, OnItemRotate)
	ON_COMMAND(ID_ITEM_TEAPOT, OnItemTeapot)
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_UPDATE_COMMAND_UI(ID_ITEM_ANIMATE, OnUpdateItemAnimate)
	ON_UPDATE_COMMAND_UI(ID_ITEM_ROTATE, OnUpdateItemRotate)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COpenGLView construction/destruction

COpenGLView::COpenGLView() :
	 m_hRC(0), m_pDC(0), m_ErrorString(_ErrorStrings[0])

{
	// TODO: add construction code here
	DisableAllItems();
	bSphere=true;
	bAnimate=true;
	bRotate=true;
	start=clock();
	finish=clock();
	b_mouseDown=false;
	b_mouseMoving=false;
	d_yAngle=d_xAngle=0.0;
	d_xDisp=d_zDisp=0.0;
	

}

COpenGLView::~COpenGLView()
{
}

BOOL COpenGLView::PreCreateWindow(CREATESTRUCT& cs) 
{
	// TODO: Add your specialized code here and/or call the base class

	// An OpenGL window must be created with the following flags and must not
    // include CS_PARENTDC for the class style. 
    cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
  	
	return CView::PreCreateWindow(cs);
}


/////////////////////////////////////////////////////////////////////////////
// COpenGLView drawing


void COpenGLView::OnDraw(CDC* pDC)
{
	COpenGLViewClassDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here

	::glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	static char s[100];
	

	static frames=0;
	frames++;
	if(frames==20){
		double  duration;
		finish = clock();
		duration = (double)(finish - start) / CLOCKS_PER_SEC;
		start=clock();
		sprintf(s,"OpenGL Demo [%3.2lf Frames Per sec]",20/duration);
		frames=0;
	}
	//pDC->DrawText(s,CRect(10,10,200,50),0);
	this->GetParent()->SetWindowText(s);


	PreRenderScene();


	::glPushMatrix();
	RenderScene();
	::glPopMatrix();

	::glFinish();

	if ( FALSE == ::SwapBuffers( m_pDC->GetSafeHdc() ) )
		{
		SetError(7);
		}
	if(bAnimate||bRotate){
		InvalidateRect(0,FALSE);
		GetParent()->PostMessage(WM_PAINT);
	}
	

}


/////////////////////////////////////////////////////////////////////////////
// COpenGLView diagnostics

#ifdef _DEBUG
void COpenGLView::AssertValid() const
{
	CView::AssertValid();
}

void COpenGLView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

COpenGLViewClassDoc* COpenGLView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(COpenGLViewClassDoc)));
	return (COpenGLViewClassDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// COpenGLView message handlers

int COpenGLView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// TODO: Add your specialized creation code here

	InitializeOpenGL();

	return 0;
}


/////////////////////////////////////////////////////////////////////////////
// GL helper functions

void COpenGLView::SetError( int e )
{
	// if there was no previous error,
	// then save this one
	if ( _ErrorStrings[0] == m_ErrorString ) 
		{
		m_ErrorString = _ErrorStrings[e];
		}
}


BOOL COpenGLView::InitializeOpenGL()
{
	// Can we put this in the constructor?
    m_pDC = new CClientDC(this);

    if ( NULL == m_pDC ) // failure to get DC
		{
		SetError(1);
		return FALSE;
		}

	if (!SetupPixelFormat())
		{
        return FALSE;
		}

    //n = ::GetPixelFormat(m_pDC->GetSafeHdc());
    //::DescribePixelFormat(m_pDC->GetSafeHdc(), n, sizeof(pfd), &pfd);

  //  CreateRGBPalette();

    if ( 0 == (m_hRC = ::wglCreateContext( m_pDC->GetSafeHdc() ) ) )
		{
		SetError(4);
		return FALSE;
		}

    if ( FALSE == ::wglMakeCurrent( m_pDC->GetSafeHdc(), m_hRC ) )
		{
		SetError(5);
		return FALSE;
		}	

	// specify black as clear color
    ::glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
	// specify the back of the buffer as clear depth
    ::glClearDepth( 1.0f );
	// enable depth testing
    ::glEnable( GL_DEPTH_TEST );

	return TRUE;
}


BOOL COpenGLView::SetupPixelFormat()
{
  static PIXELFORMATDESCRIPTOR pfd = 
	{
        sizeof(PIXELFORMATDESCRIPTOR),  // size of this pfd
        1,                              // version number
        PFD_DRAW_TO_WINDOW |            // support window
          PFD_SUPPORT_OPENGL |          // support OpenGL
          PFD_DOUBLEBUFFER,             // double buffered
        PFD_TYPE_RGBA,                  // RGBA type
        24,                             // 24-bit color depth
        0, 0, 0, 0, 0, 0,               // color bits ignored
        0,                              // no alpha buffer
        0,                              // shift bit ignored
        0,                              // no accumulation buffer
        0, 0, 0, 0,                     // accum bits ignored
//        32,                             // 32-bit z-buffer
		16,	// NOTE: better performance with 16-bit z-buffer
        0,                              // no stencil buffer
        0,                              // no auxiliary buffer
        PFD_MAIN_PLANE,                 // main layer
        0,                              // reserved
        0, 0, 0                         // layer masks ignored
    };
    int pixelformat;

    if ( 0 == (pixelformat = ::ChoosePixelFormat(m_pDC->GetSafeHdc(), &pfd)) )
	    {
		SetError(2);
        return FALSE;
		}

    if ( FALSE == ::SetPixelFormat(m_pDC->GetSafeHdc(), pixelformat, &pfd) )
	    {
       	SetError(3);
        return FALSE;
		}

    return TRUE;
}

void COpenGLView::OnDestroy() 
{
	CView::OnDestroy();
	
	// TODO: Add your message handler code here

    if ( FALSE == ::wglDeleteContext( m_hRC ) )
		{
		SetError(6);
 		}

    if ( m_pDC )
		{
        delete m_pDC;
		}
}


BOOL COpenGLView::OnEraseBkgnd( CDC* pDC ) 
{
	// TODO: Add your message handler code here and/or call default
	
	//	return CView::OnEraseBkgnd(pDC);
	return TRUE; // tell Windows not to erase the background
}

void COpenGLView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	// TODO: Add your message handler code here
	GLdouble aspect_ratio; // width/height ratio
	
	if ( 0 >= cx || 0 >= cy )
		{
		return;
		}


	SetupViewport( cx, cy );

	// select the projection matrix and clear it
    ::glMatrixMode( GL_PROJECTION );
    ::glLoadIdentity();

	// compute the aspect ratio
	// this will keep all dimension scales equal
	aspect_ratio = (GLdouble)cx/(GLdouble)cy;

	// select the viewing volumn
	SetupViewingFrustum( aspect_ratio );
	
	// switch back to the modelview matrix
    ::glMatrixMode( GL_MODELVIEW );
    ::glLoadIdentity();

	// now perform any viewing transformations
	SetupViewingTransform();
}

 /////////////////////////////////////////////////////////////////////////////
// COpenGLView helper functions

BOOL COpenGLView::SetupViewport( int cx, int cy )
{
	// select the full client area
    ::glViewport(0, 0, cx, cy);

	return TRUE;
}

BOOL COpenGLView::SetupViewingFrustum( GLdouble aspect_ratio )
{
	// select a default viewing volumn
    ::gluPerspective( 40.0f, aspect_ratio, .1f, 20.0f );
	return TRUE;
}


BOOL COpenGLView::SetupViewingTransform()
{
	// select a default viewing transformation
	// of a 20 degree rotation about the X axis
	// then a -5 unit transformation along Z
	::glTranslatef( 0.0f, 0.0f, -5.0f );
	::glRotatef( 20.0f, 1.0f, 0.0f, 0.0f );
    return TRUE;
}


BOOL COpenGLView::RenderScene()
{
	if(bRotate && !b_mouseDown){
		d_xAngle+=1.0f;
	}
	::glRotatef( (float)d_xAngle, (float)1.0f, (float)0.0f, 0.0f );

	::glRotatef( (float)d_yAngle, (float)0.0f, (float)0.0f, 1.0f );
	::glTranslatef( (float)d_xDisp, (float)0.0f, (float)d_zDisp);
	
	::glColor3f( 1.0f,1.0f,1.0f);

	if(bCylinder)RenderCylinder();
	if(bSphere)RenderSphere();
	if(bTorus)RenderTorus();
	if(bCoil)RenderCoil();
	if(bBiLinear)RenderBiLinear();
	if(bGoblet)RenderGoblet();
	if(bEllipsoid) RenderEllipsoid();
	if(bHyperboloid) RenderHyperboloid();
	if(bSuperEllipsoid) RenderSuperEllipsoid();
	if(bSuperToroid) RenderSuperToroid();
 	if(bTeapot) RenderTeapot();
    return TRUE;
}	

void COpenGLView::RenderCylinder(){
	GLfloat vertex1[3];
	GLfloat vertex2[3];
	GLfloat vertex3[3];

	float u,v, delta_u=0.02f,delta_v=0.02f;

	//::glColor3f( 1.0f, 0.0f, 0.0f ); //rgb; 1,0,0 = red
	::glBegin(GL_LINES); // each pair of glVertex will form a line

	// draw body of cylinder
	for(u=0;u<1;u+=delta_u){
		for(v=0;v<1;v+=delta_v){
			MyCylinder(u,v,vertex1);
			MyCylinder(u+delta_u,v,vertex2);
			glVertex3fv(vertex1);
			glVertex3fv(vertex2);
			MyCylinder(u,v+delta_v,vertex3);
			glVertex3fv(vertex1);
			glVertex3fv(vertex3);
		}
	}
	for(v=0;v<1;v+=delta_v){ //this loop draws the last circle on the cylinder
		MyCylinder(u,v,vertex1);
		MyCylinder(u,v+delta_v,vertex2);
		glVertex3fv(vertex1);
		glVertex3fv(vertex2);
		}

	::glEnd();
}

#define TWO_PI 2*3.141

void COpenGLView::MyCylinder(float u,float v,GLfloat fv[]){
	//implementation of paramentric function of a cylinder
	// the line px->py is rotated around the y-axis
	float p1x=1.0f,p1y=1.0f;
	float p2x=0.5f,p2y=-1.0f;

	fv[0]=(p1x+(p2x-p1x)*u)*(float)cos(TWO_PI*v);	//y-coordinate
	fv[1]=p1y+(p2y-p1y)*u;							//y-coordinate
	fv[2]=(p1x+(p2x-p1x)*u)*(float)sin(TWO_PI*v);	//z-coordinate
}



void COpenGLView::RenderSphere(){
	GLfloat vertex1[3];
	GLfloat vertex2[3];
	GLfloat vertex3[3];

	float u,v, delta_u=0.02f,delta_v=0.02f;

	//::glColor3f( 1.0f, 0.0f, 0.0f ); //rgb; 1,0,0 = red
	::glBegin(GL_LINES); // each pair of glVertex will form a line

	// draw body of cylinder
	for(u=0;u<1;u+=delta_u){
		for(v=0;v<1;v+=delta_v){
			MySphere(u,v,vertex1);
			MySphere(u+delta_u,v,vertex2);
			glVertex3fv(vertex1);
			glVertex3fv(vertex2);
			MySphere(u,v+delta_v,vertex3);
			glVertex3fv(vertex1);
			glVertex3fv(vertex3);
		}
	}

	::glEnd();
}


void COpenGLView::MySphere(float u,float v,GLfloat fv[]){

	float r=1.0f;
	float x_u=r*(float)cos(TWO_PI*u);

	fv[0]=x_u*(float)cos(TWO_PI*v);	//x-coordinate
	fv[1]=r*(float)sin(TWO_PI*u);	//y-coordinate
	fv[2]=x_u*(float)sin(TWO_PI*v);	//z-coordinate
}


void COpenGLView::RenderTorus(){
	GLfloat vertex1[3];
	GLfloat vertex2[3];
	GLfloat vertex3[3];

	float u,v, delta_u=0.02f,delta_v=0.02f;

	//::glColor3f( 1.0f, 0.0f, 0.0f ); //rgb; 1,0,0 = red
	::glBegin(GL_LINES); // each pair of glVertex will form a line

	// draw body of cylinder
	for(u=0;u<1;u+=delta_u){
		for(v=0;v<1;v+=delta_v){
			MyTorus(u,v,vertex1);
			MyTorus(u+delta_u,v,vertex2);
			glVertex3fv(vertex1);
			glVertex3fv(vertex2);
			MyTorus(u,v+delta_v,vertex3);
			glVertex3fv(vertex1);
			glVertex3fv(vertex3);
		}
	}

	::glEnd();
}


void COpenGLView::MyTorus(float u,float v,GLfloat fv[]){

	float r=0.4f;
	float R=1.3f;
	float x_u=r*(float)cos(TWO_PI*u)+R;

	fv[0]=x_u*(float)cos(TWO_PI*v);	//x-coordinate
	fv[1]=r*(float)sin(TWO_PI*u);	//y-coordinate
	fv[2]=x_u*(float)sin(TWO_PI*v);	//z-coordinate
}

void COpenGLView::RenderCoil(){
	GLfloat vertex1[3];
	GLfloat vertex2[3];
	GLfloat vertex3[3];

	float u,v, delta_u=0.05f,delta_v=0.05f;

//	::glColor3f( 1.0f, 0.0f, 0.0f ); //rgb; 1,0,0 = red
	::glBegin(GL_LINES); // each pair of glVertex will form a line

	// draw body of cylinder
	for(u=0;u<1;u+=delta_u){
		for(v=-2;v<5;v+=delta_v){
			MyCoil(u,v,vertex1);
			MyCoil(u+delta_u,v,vertex2);
			glVertex3fv(vertex1);
			glVertex3fv(vertex2);
			MyCoil(u,v+delta_v,vertex3);
			glVertex3fv(vertex1);
			glVertex3fv(vertex3);
		}
	}

	::glEnd();
}


void COpenGLView::MyCoil(float u,float v,GLfloat fv[]){

	float r=0.2f;
	float R=1.1f;
	float x_u=r*(float)cos(TWO_PI*u)+R;

	fv[0]=x_u*(float)cos(TWO_PI*v);	//x-coordinate
	fv[1]=r*(float)sin(TWO_PI*u)-1.5f+v;	//y-coordinate
	fv[2]=x_u*(float)sin(TWO_PI*v);	//z-coordinate
}

void COpenGLView::RenderGoblet(){
	GLfloat vertex1[3];
	GLfloat vertex2[3];
	GLfloat vertex3[3];

	float v, delta_u=0.05f,delta_v=0.05f;
	int u;

//	::glColor3f( 1.0f, 0.0f, 0.0f ); //rgb; 1,0,0 = red
	::glScalef(2.0f,2.0f,2.0f);
	::glBegin(GL_LINES); // each pair of glVertex will form a line

	// draw body of cylinder
	for(u=0;u<19;u+=1){
		for(v=0;v<1;v+=delta_v){
			MyGoblet(u,v,vertex1);
			MyGoblet(u+1,v,vertex2);
			glVertex3fv(vertex1);
			glVertex3fv(vertex2);
			MyGoblet(u,v+delta_v,vertex3);
			glVertex3fv(vertex1);
			glVertex3fv(vertex3);
		}
	}

	::glEnd();
}


void COpenGLView::MyGoblet(int u,float v,GLfloat fv[]){

	float pt[][3]={		{0.0f,0.26f},//1
						{0.1f,0.265f},//2
						{0.17f,0.3f},//3
						{0.22f,0.35f},//4
						{0.235f,0.4f},//5
						{0.24f,0.5f},//6
						{0.25f,0.5f},//7
						{0.245f,0.395f},//8
						{0.23f,0.345f},//9
						{0.175f,0.29f},//10
						{0.1f,0.255f},//11
						{0.02f,0.25f},//12
						{0.02f,0.15f},//13
						{0.03f,0.13f},//14
						{0.03f,0.12f},//15
						{0.02f,0.10f},//16
						{0.02f,0.03f},//17
						{0.20f,0.00f},//18
						{0.00f,0.00f}//19
	};
	u=u%19;
	fv[0]=pt[u][0]*(float)cos(TWO_PI*v);
	fv[1]=pt[u][1];
	fv[2]=pt[u][0]*(float)sin(TWO_PI*v);
}

void COpenGLView::RenderBiLinear(){
	GLfloat vertex1[3];
	GLfloat vertex2[3];
	GLfloat vertex3[3];

	float u,v, delta_u=0.05f,delta_v=0.05f;

//	::glColor3f( 1.0f, 0.0f, 0.0f ); //rgb; 1,0,0 = red
	::glBegin(GL_LINES); // each pair of glVertex will form a line

	// draw body of patch
	for(u=0;u<1;u+=delta_u){
		for(v=0;v<1;v+=delta_v){
			MyBiLinear(u,v,vertex1);
			MyBiLinear(u+delta_u,v,vertex2);
			glVertex3fv(vertex1);
			glVertex3fv(vertex2);
			MyBiLinear(u,v+delta_v,vertex3);
			glVertex3fv(vertex1);
			glVertex3fv(vertex3);
		}
	}

	::glEnd();
}


void COpenGLView::MyBiLinear(float u,float v,GLfloat fv[]){

	float p0[3]={-1.0f,2.0f,0.0f};
	float p1[3]={-1.0f,-2.0f,0.0f};
	float p2[3]={1.0f,0.0f,-2.0f};
	float p3[3]={1.0f,0.0f,+1.0f};

	fv[0]=(1-v)*(1-u)*p0[0]+(1-v)*u*p1[0]+v*(1-u)*p2[0]+v*u*p3[0];
	fv[1]=(1-v)*(1-u)*p0[1]+(1-v)*u*p1[1]+v*(1-u)*p2[1]+v*u*p3[1];
	fv[2]=(1-v)*(1-u)*p0[2]+(1-v)*u*p1[2]+v*(1-u)*p2[2]+v*u*p3[2];
}

void COpenGLView::RenderEllipsoid(){
	GLfloat vertex1[3];
	GLfloat vertex2[3];
	GLfloat vertex3[3];

	float u,v, delta_u=0.05f,delta_v=0.05f;

	static float t=0;
	static BOOL growing=FALSE;

	if(bAnimate){
		if(growing )
			t+=0.01f;
		else 
			t-=0.01f;
		if(t>0.5)
			growing=FALSE;
		if(t<-0.5)
			growing=TRUE;
	}

//	::glColor3f( 1.0f, 0.0f, 0.0f ); //rgb; 1,0,0 = red
	::glBegin(GL_LINES); // each pair of glVertex will form a line

	// draw body of Ellipsoid
		for(u=-0.5;u<0.5;u+=delta_u){
			for(v=-0.25;v<0.25;v+=delta_v){
				MyEllipsoid(u,v,t,vertex1);
				MyEllipsoid(u+delta_u,v,t,vertex2);
				glVertex3fv(vertex1);
				glVertex3fv(vertex2);
				MyEllipsoid(u,v+delta_v,t,vertex3);
				glVertex3fv(vertex1);
				glVertex3fv(vertex3);
			}
		}

	::glEnd();
}


void COpenGLView::MyEllipsoid(float u,float v,float t,GLfloat fv[]){

	float x_v=(float)cos(TWO_PI*v);

	fv[0]=(1.0f+t)*x_v*(float)cos(TWO_PI*u);	//x-coordinate
	fv[1]=(1.0f+t)*x_v*(float)sin(TWO_PI*u);	//y-coordinate
	fv[2]=(1.0f-t)*(float)sin(TWO_PI*v);	//z-coordinate
}

void COpenGLView::RenderSuperEllipsoid(){
	GLfloat vertex1[3];
	GLfloat vertex2[3];
	GLfloat vertex3[3];

	//CDC *pdc;
	//pdc=CDC::FromHandle(wglGetCurrentDC());
	//pdc->DrawText("Hello",CRect(10,10,50,50),0);

	float u,v, delta_u=0.05f,delta_v=0.05f;
	::glRotatef(45.00,0.0,1.0,0.0f);

//	::glColor3f( 1.0f, 0.0f, 0.0f ); //rgb; 1,0,0 = red
	::glBegin(GL_LINES); // each pair of glVertex will form a line

	// draw body of SuperEllipsoid
	for(u=-0.5;u<=0.6;u+=delta_u){
		for(v=-0.25;v<=0.25;v+=delta_v){
			MySuperEllipsoid(u,v,vertex1);
			MySuperEllipsoid(u+delta_u,v,vertex2);
			glVertex3fv(vertex1);
			glVertex3fv(vertex2);
			MySuperEllipsoid(u,v+delta_v,vertex3);
			glVertex3fv(vertex1);
			glVertex3fv(vertex3);
		}
	}

	::glEnd();


}


void COpenGLView::MySuperEllipsoid(float u,float v,GLfloat fv[]){

	static double m=10.0,n;
	double delta_m=0;
	static BOOL growing=FALSE;

	delta_m=sqrt(m/100)/10000;

	if(m<0.5)
		growing=TRUE;
	if(m>10.0)
		growing=FALSE;

	if(bAnimate){
		if(growing)
			m+=delta_m;
		else
			m-=delta_m;
	}
	n=m;

	fv[0]=(float)(cos(TWO_PI*v)*pow(sqrt(cos(TWO_PI*v)*cos(TWO_PI*v)),((2.0/m)-1.0))*
				cos(TWO_PI*u)*pow(sqrt(cos(TWO_PI*u)*cos(TWO_PI*u)),((2.0/n)-1.0)));	//x-coordinate

	fv[1]=(float)(cos(TWO_PI*v)*pow(sqrt(cos(TWO_PI*v)*cos(TWO_PI*v)),((2.0/m)-1.0))*
				sin(TWO_PI*u)*pow(sqrt(sin(TWO_PI*u)*sin(TWO_PI*u)),((2.0/n)-1.0)));	//x-coordinate

	fv[2]=(float)(sin(TWO_PI*v)*pow(sqrt(sin(TWO_PI*v)*sin(TWO_PI*v)),((2.0/m)-1.0)));	//x-coordinate
}

void COpenGLView::RenderHyperboloid(){
	GLfloat vertex1[3];
	GLfloat vertex2[3];
	GLfloat vertex3[3];

	float u,v, delta_u=0.02f,delta_v=0.02f;

//	::glColor3f( 1.0f, 0.0f, 0.0f ); //rgb; 1,0,0 = red
	::glScalef(0.5f,0.5f,0.5f);
	::glBegin(GL_LINES); // each pair of glVertex will form a line

	// draw body of cylinder
	for(u=-0.5f;u<0.5f;u+=delta_u){
		for(v=-0.20f;v<0.0f;v+=delta_v){
			MyHyperboloid(u,v,vertex1);
			MyHyperboloid(u+delta_u,v,vertex2);
			glVertex3fv(vertex1);
			glVertex3fv(vertex2);
			MyHyperboloid(u,v+delta_v,vertex3);
			glVertex3fv(vertex1);
			glVertex3fv(vertex3);
		}
	}

	::glEnd();
}


void COpenGLView::MyHyperboloid(float u,float v,GLfloat fv[]){

	float x_v=1.0f/((float)cos(TWO_PI*v));

	fv[0]=x_v*(float)cos(TWO_PI*u);	//x-coordinate
	fv[1]=x_v*(float)sin(TWO_PI*u);	//y-coordinate
	fv[2]=(float)tan(TWO_PI*v);	//z-coordinate
}

void COpenGLView::RenderSuperToroid(){
	GLfloat vertex1[3];
	GLfloat vertex2[3];
	GLfloat vertex3[3];

	float u,v, delta_u=0.05f,delta_v=0.05f;
	::glScalef(0.5f,0.5f,0.5f);
//	::glColor3f( 1.0f, 0.0f, 0.0f ); //rgb; 1,0,0 = red
	::glBegin(GL_LINES); // each pair of glVertex will form a line

	// draw body of cylinder
	for(u=-0.5;u<=0.6;u+=delta_u){
		for(v=-0.5;v<=0.6;v+=delta_v){
			MySuperToroid(u,v,vertex1);
			MySuperToroid(u+delta_u,v,vertex2);
			glVertex3fv(vertex1);
			glVertex3fv(vertex2);
			MySuperToroid(u,v+delta_v,vertex3);
			glVertex3fv(vertex1);
			glVertex3fv(vertex3);
		}
	}

	::glEnd();
}


void COpenGLView::MySuperToroid(float u,float v,GLfloat fv[]){

	static double m=10.0,n,d=2.0;
	double delta_m=0;
	static BOOL growing=FALSE;
	static long steady=0;

	delta_m=sqrt(m/1000)/10000;

	if(m<0.5){
		growing=TRUE;
		steady=200000;
	}
	if(m>10.0)
		growing=FALSE;

	
	if(growing)
		m+=delta_m;
	else
		m-=delta_m;

	if(steady > 0){
		m=0.5;
		steady--;
	}

	n=m;

	fv[0]=(float)((d+cos(TWO_PI*v)*pow(sqrt(cos(TWO_PI*v)*cos(TWO_PI*v)),((2.0/m)-1.0)))*
				cos(TWO_PI*u)*pow(sqrt(cos(TWO_PI*u)*cos(TWO_PI*u)),((2.0/n)-1.0)));	//x-coordinate

	fv[1]=(float)((d+cos(TWO_PI*v)*pow(sqrt(cos(TWO_PI*v)*cos(TWO_PI*v)),((2.0/m)-1.0)))*
				sin(TWO_PI*u)*pow(sqrt(sin(TWO_PI*u)*sin(TWO_PI*u)),((2.0/n)-1.0)));	//x-coordinate

	fv[2]=(float)(sin(TWO_PI*v)*pow(sqrt(sin(TWO_PI*v)*sin(TWO_PI*v)),((2.0/m)-1.0)));	//x-coordinate
}

void COpenGLView::RenderTeapot(){
	::auxWireTeapot(1.0);
}
void COpenGLView::DisableAllItems(){
	bTorus=bCylinder=bGoblet=bSphere=bCoil=bGoblet=bBiLinear=FALSE;
	bEllipsoid=bHyperboloid=bSuperEllipsoid=bSuperToroid=bTeapot=FALSE;
}

void COpenGLView::OnItemTorus() 
{
	// TODO: Add your command handler code here
	DisableAllItems();
	bTorus=true;
	/*	if(bAnimated){
		InvalidateRect(0,FALSE);
		GetParent()->PostMessage(WM_PAINT);
	}
*/
		InvalidateRect(0,FALSE);
		GetParent()->PostMessage(WM_PAINT);
	
}

void COpenGLView::OnItemSphere() 
{
	// TODO: Add your command handler code here
	DisableAllItems();
	bSphere=true;
	InvalidateRect(0,FALSE);

	
}

void COpenGLView::OnItemCoil() 
{
	// TODO: Add your command handler code here
	DisableAllItems();
	bCoil=true;
	InvalidateRect(0,FALSE);
	
}

void COpenGLView::OnItemGoblet() 
{
	// TODO: Add your command handler code here
	DisableAllItems();
	bGoblet=true;
	InvalidateRect(0,FALSE);

	
}

void COpenGLView::OnItemCylinder() 
{
	// TODO: Add your command handler code here
	DisableAllItems();
	bCylinder=true;
	InvalidateRect(0,FALSE);
	
}

void COpenGLView::OnItemBilinear() 
{
	// TODO: Add your command handler code here
	DisableAllItems();
	bBiLinear=true;
	InvalidateRect(0,FALSE);
	
}

void COpenGLView::OnQuadricsEllipseoid() 
{
	// TODO: Add your command handler code here
	DisableAllItems();
	bEllipsoid=true;
	InvalidateRect(0,FALSE);
	
}

void COpenGLView::OnQuadricsHyperboloid() 
{
	// TODO: Add your command handler code here
	DisableAllItems();
	bHyperboloid=true;
	InvalidateRect(0,FALSE);
	
}

void COpenGLView::OnSuperquadricsSuperellipseoid() 
{
	// TODO: Add your command handler code here
	DisableAllItems();
	bSuperEllipsoid=true;
	InvalidateRect(0,FALSE);
	
}

void COpenGLView::OnSuperquadricsSupertoroid() 
{
	// TODO: Add your command handler code here
	DisableAllItems();
	bSuperToroid=true;
	InvalidateRect(0,FALSE);
	
}

void COpenGLView::OnItemAnimate() 
{
	// TODO: Add your command handler code here
	bAnimate=(bAnimate==TRUE)?FALSE:TRUE;
	InvalidateRect(0,FALSE);

	
}

void COpenGLView::OnItemRotate() 
{
	// TODO: Add your command handler code here
	bRotate=(bRotate==TRUE)?FALSE:TRUE;
	InvalidateRect(0,FALSE);

	
}

void COpenGLView::OnItemTeapot() 
{
	// TODO: Add your command handler code here
	DisableAllItems();
	bTeapot=true;
	InvalidateRect(0,FALSE);
	
}

void COpenGLView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	CView::OnLButtonDown(nFlags, point);

	b_mouseDown=true;
}

void COpenGLView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	CView::OnMouseMove(nFlags, point);

	if(b_mouseDown){
		if(nFlags&MK_CONTROL){
			d_xDisp+=(float)(point.y-lastpoint.y)/100.0;
			d_zDisp+=(float)(point.y-lastpoint.y)/100.0;
		}
		else{
			d_xAngle+=point.y-lastpoint.y;
			d_yAngle-=point.x-lastpoint.x;
		}
		InvalidateRect(0,FALSE);
		GetParent()->PostMessage(WM_PAINT);

	}
	
	lastpoint=point;

}

void COpenGLView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	CView::OnLButtonUp(nFlags, point);
	b_mouseDown=false;
	b_mouseMoving=false;
}

void COpenGLView::OnUpdateItemAnimate(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
		pCmdUI->SetCheck(bAnimate);

	
}

void COpenGLView::OnUpdateItemRotate(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
		pCmdUI->SetCheck(bRotate);

	
}
